(function(angular, $, _) {

	angular.module('form_processor').directive('crmFormProcessorActions', function(formProcessorFactory, dialogService, $timeout) {
	  return {
	    restrict: 'E',
	    scope: {
	    	inputs: '=',
	      actions: '=',
	      actionTitles: '=',
	      formProcessor: '=',
	      deletedActions: '=',
	      context: '@',
	      entity: '@',
				enableDelay: '@',
        addButtonLabel: '@',
        addDropdownLabel: '@',
        dialogTitle: '@'
	    },
	    templateUrl: '~/form_processor/actions/crmFormProcessorActions.html',
	    link: function($scope, element, attrs){
	    	$scope.ts = CRM.ts(null);
				var delayHandlers = CRM.form_processor.delayedActionHandlers;
		    for(var i=0; i<$scope.actions.length; i++) {
		    	$scope.actions[i].delay_title = null;
		    	if ($scope.actions[i].delay) {
						angular.forEach(delayHandlers, function (delayedActionHandler, key) {
							if (delayedActionHandler.name == $scope.actions[i].delay) {
								$scope.actions[i].delay_title = delayedActionHandler.title;
							}
						});
					}
				}

	    	$scope.new_id = -1;

	    	$timeout(function(){
		    	// The select boxes for input type and action type do not count for the @pristine state of the
		    	// form.
		  		$scope.formProcessorActionsForm.type.$pristine = false;
				});

	    	$scope.editCondition = function editCondition(action) {
          var options = CRM.utils.adjustDialogDefaults({
            autoOpen: false,
            width: '40%',
            height: '80%',
            title: $scope.dialogTitle ? $scope.dialogTitle : ts('Edit condition')
          });

          formProcessorFactory.getFieldList($scope.inputs, $scope.actions, action, $scope.context)
					.then(function (fields) {
						var index = $scope.actions.indexOf(action);
						var model = {
							action: angular.copy(action),
							fields: fields,
              context: $scope.context
						};
						dialogService.open('ConditionDialog', '~/form_processor/actions/ConditionDialogCtrl.html', model, options)
							.then(function(data) {
								$scope.formProcessorActionsForm.$setDirty();
								if (index >= 0) {
									$scope.actions[index].condition_configuration = angular.copy(data.action.condition_configuration);
								}
							});
					});
				};

				$scope.editDelay = function editDelay(action) {
					var options = CRM.utils.adjustDialogDefaults({
						autoOpen: false,
						width: '40%',
						height: '80%',
						title: ts('Edit delay')
					});

					formProcessorFactory.getFieldList($scope.inputs, $scope.actions, action, $scope.context)
							.then(function (fields) {
								var index = $scope.actions.indexOf(action);
								var model = {
									action: angular.copy(action),
									fields: fields,
									context: $scope.context,
									delayedActionHandlers: CRM.form_processor.delayedActionHandlers
								};
								dialogService.open('DelayedActionHandlerDialog', '~/form_processor/DelayedActionHandlerDialogCtrl.html', model, options)
										.then(function(data) {
											$scope.formProcessorActionsForm.$setDirty();
											if (index >= 0) {
												$scope.actions[index].delay = angular.copy(data.action.delay);
												$scope.actions[index].delay_title = angular.copy(data.action.delay_title);
												$scope.actions[index].delay_configuration = angular.copy(data.action.delay_configuration);
											}
										});
							});
				};

	    	$scope.removeAction = function removeAction(action) {
		    	var index = $scope.actions.indexOf(action);
		    	if (index >= 0) {
		    		$scope.actions.splice(index, 1);
		    		$scope.deletedActions.push(action);
		    	}
		    	$scope.formProcessorActionsForm.$setDirty();
		    };

		   	// Open a dialog for adding an action
		    $scope.addAction = function addAction(actionType) {
		    	if (!actionType) {
		    		return;
		    	}

		    	var action = {
						'id': $scope.new_id,
						'type': actionType,
						'title': '',
						'name': '',
						'mapping': {}
					};
					$scope.editAction(action);
		    };

		    // Open a dialog for editting an action
		    $scope.editAction = function editAction(action) {
		      var options = CRM.utils.adjustDialogDefaults({
		        autoOpen: false,
		        width: '40%',
		        height: '80%',
		        title: ts('Edit action')
		      });

          index = $scope.actions.indexOf(action);
          formProcessorFactory.getAction(action.type, $scope.context).
          then(function(actionObject) {
            if (index < 0) {
              action.configuration = angular.copy(actionObject.default_configuration);
            }
            formProcessorFactory.getFieldList($scope.inputs, $scope.actions, actionObject, $scope.context).
            then(function (fields) {
              var model = {
                formProcessor: $scope.formProcessor,
                action: action,
								actions: $scope.actions,
                actionType: actionObject,
                fields: fields,
                entity: $scope.entity,
                context: $scope.context
              };
              dialogService.open('ActionDialog', '~/form_processor/actions/ActionDialogCtrl.html', model, options).
              then(function (data) {
                $scope.formProcessorActionsForm.$setDirty();
                if (index >= 0) {
                  $scope.actions[index] = data.action;
                }
                else if ($scope.new_id == data.action.id) {
                  $scope.actions.push(data.action);
                  $scope.new_id--;
                }
              });
            });
          });
		    };
	    }
	  };
	});

})(angular, CRM.$, CRM._);
